﻿//================================================================================
//  
//        APB-2_Form.cs        ＡＰＢ－２　テストプログラム
//  
//        (c)2011/07/07  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなどの
//  商業的利用はおやめください。
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Management;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace APB_2
{
    public partial class Form1 : Form
    {
        private SerialPort port = new SerialPort();
        private string[] PortList;
        private volatile string msg = "";       // 別スレッドで変更される

        private int filesize = 0;               // ファイルサイズ
        private byte[] buf = new byte[1];       // ファイル読み込みデータ格納用配列


        public Form1()
        {
            InitializeComponent();

            port.DataReceived += new SerialDataReceivedEventHandler(port_DataReceived); // シリアル受信ハンドラー

            ListComport();                      // 接続されているシリアルポート

            richTextBox1.Font = new Font("ＭＳ ゴシック", richTextBox1.SelectionFont.Size); // 等幅フォント
        }


        ~Form1()
        {
            if( port.IsOpen ) port.Close();
        }


        //
        //　　接続されているシリアルポートのリストアップ
        //
        private void ListComport()
        {
            // プロジェクトメニューから System.Management の参照の追加が必要

            PortList = SerialPort.GetPortNames();
            ManagementObjectSearcher searcher = new ManagementObjectSearcher("SELECT * FROM WIN32_SerialPort");
            var ports = searcher.Get().Cast<ManagementBaseObject>().ToList();
            var tList = (from n in PortList join p in ports on n equals p["DeviceID"].ToString() select p["Caption"]).ToList();    // 詳細なシリアルポート名を得る
            comboBox_port.Items.Clear();
            foreach (string s in tList)  comboBox_port.Items.Add(s);
            if (comboBox_port.Items.Count > 0) comboBox_port.SelectedIndex = comboBox_port.Items.Count - 1; // 1番大きいCOM番号
        }


        //
        //　　シリアル受信 イベントハンドラー
        //
        private void port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            if (port.IsOpen)
            {
                msg = port.ReadExisting();
                richTextBox1.Invoke(new EventHandler(delegate
                {
                    richTextBox1.SelectionStart = richTextBox1.Text.Length;
                    richTextBox1.SelectedText = msg;
                    richTextBox1.ScrollToCaret();
                }));
            }
        }


        //
        //　　スキャンシリアルポートボタン クリックイベントハンドラー
        //
        private void button_ScanCOM_Click(object sender, EventArgs e)
        {
            ListComport();                          // 接続されているシリアルポートを再度調べる
        }

        //
        //　　シリアルポートオープンボタン クリックイベントハンドラー
        //
        private void button_OpenCOM_Click(object sender, EventArgs e)
        {
            try
            {
                if (port.IsOpen) port.Close();      // 前にオープンしていたらクローズ

                port.PortName = PortList[comboBox_port.SelectedIndex];
                port.Open();
            }
            catch (System.Exception ex)
            {
                Print( "\rERROR:  Can't open Com port\r" );
            }

            if (port.IsOpen)
            {
                Print( "\rOpen " + port.PortName + "\r" );
            }
        }


        //
        //　　バージョン表示ボタン クリックイベントハンドラー
        //
        private void button_Version_Click(object sender, EventArgs e)
        {
            PortWrite("\r");                        // 未定義コマンドを送るとバージョンとヘルプ表示
        }


        //
        //　　ＤａｔＦｌａｓｈ ＩＤ 読み出しボタン クリックイベントハンドラー
        //
        private void button_DataFlashID_Click(object sender, EventArgs e)
        {
            PortWrite("d" + "\r");                  // データフラッシュＩＤ読み出しコマンド送信
        }


        //
        //　　ＤａｔＦｌａｓｈ 読み出しボタン クリックイベントハンドラー
        //
        private void button_DataFLashReadPage_Click(object sender, EventArgs e)
        {
            PortWrite("dr" + textBox_page.Text + "\r");
        }


        //
        //　　ＤａｔＦｌａｓｈ プログラムボタン クリックイベントハンドラー
        //
        private void button_DataFLashProgram_Click(object sender, EventArgs e)
        {
            if (port.IsOpen)
            {
                PortWrite("dp");                     // データフラッシュプログラムコマンド送信

                OpenFile();
                if (filesize != 0)
                {
                    Thread t = new Thread(new ThreadStart(tsenddata));
                    t.Start();                      // 別スレッドでファイル転送する
                }
                else
                {
                    PortWrite("\r");                // キャンセルしてもデータフラッシュは消去されてしまっている
                }
            }
        }
        
         
        //
        //　　ＦＰＧＡ コンフィグボタン クリックイベントハンドラー
        //
        private void button_FPGAConfig_Click(object sender, EventArgs e)
        {
            if (port.IsOpen)
            {
                PortWrite("c");                     // コンフィグコマンド送信
                Thread.Sleep(200);                  // 基板からの応答を待つ
                if (msg.EndsWith(">")) return;      // Master SPI モードならすでにコンフィグ終了している

                OpenFile();
                if (filesize != 0)
                {
                    Thread t = new Thread(new ThreadStart(tsenddata));
                    t.Start();                      // 別スレッドでファイル転送する
                }
                else
                {
                    PortWrite("\r");                // コンフィグキャンセル
                }
            }
        }


        //
        // ファイル転送スレッド
        //
        void tsenddata()
        {
            port.Write(buf, 0, filesize);
        }


        //
        //　　ＦＰＧＡ 書き込みボタン クリックイベントハンドラー
        //
        private void button_FPGAWrite_Click(object sender, EventArgs e)
        {
            PortWrite("w" + textBox_wadrs.Text + " " + textBox_wdata.Text + "\r");
        }


        //
        //　　ＦＰＧＡ 読み出しボタン クリックイベントハンドラー
        //
        private void button_FPGARead_Click(object sender, EventArgs e)
        {
            PortWrite("r" + textBox_radrs.Text + "\r");
        }


        //
        //　　ファイルオープンしバッファーに読み込む
        //
        private void OpenFile()
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = "";
            ofd.Filter = "bitファイル(*.bit)|*.bit|すべてのファイル(*.*)|*.*";
            ofd.FilterIndex = 1;
            ofd.Title = "書き込むファイルを選択してください";
            ofd.RestoreDirectory = true;

            filesize = 0;
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                FileStream fs = new FileStream( ofd.FileName, FileMode.Open );
                filesize = (int)fs.Length;          // ファイルサイズ
                buf = new byte[filesize];           // 
                fs.Read(buf, 0, filesize);          // 全部読み込む
                fs.Close();
                Print( ofd.FileName + "\n" ); 
            }
        }


        //
        //　　画面文字列出力
        //
        private void Print(string msg)
        {
            richTextBox1.SelectionStart = richTextBox1.Text.Length;
            richTextBox1.SelectedText = msg;
            richTextBox1.ScrollToCaret();
        }


        //
        //　　シリアルポート文字列出力
        //
        private void PortWrite(string msg)
        {
            try
            {
                if (port.IsOpen) port.Write(msg);
            }
            catch (System.Exception ex)
            {
                Print(ex.ToString());
            }
        }

    }
}
